from typing import Optional, Union

from collections import namedtuple

from vega_sim.api.market import MarketConfig
from vega_sim.environment.agent import StateAgentWithWallet
from vega_sim.network_service import VegaServiceNetwork
from vega_sim.null_service import VegaServiceNull

WALLET_NAME = "research"
WALLET_PASS = "pass"

PartyConfig = namedtuple("PartyConfig", ["wallet_name", "wallet_pass", "key_name"])

PROPOSAL_PARTY = PartyConfig(WALLET_NAME, WALLET_PASS, "proposal_party")
TERMINATION_PARTY = PartyConfig(WALLET_NAME, WALLET_PASS, "termination_party")
MAKER_PARTY = PartyConfig(WALLET_NAME, WALLET_PASS, "maker_party")
AUCTION_PARTY_A = PartyConfig(WALLET_NAME, WALLET_PASS, "auction_party_a")
AUCTION_PARTY_B = PartyConfig(WALLET_NAME, WALLET_PASS, "auction_party_b")
SENSITIVE_PARTY_A = PartyConfig(WALLET_NAME, WALLET_PASS, "sensitive_party_a")
SENSITIVE_PARTY_B = PartyConfig(WALLET_NAME, WALLET_PASS, "sensitive_party_b")
SENSITIVE_PARTY_C = PartyConfig(WALLET_NAME, WALLET_PASS, "sensitive_party_c")
INFORMED_PARTY = PartyConfig(WALLET_NAME, WALLET_PASS, "informed_party")


class ConfigurableMarketManager(StateAgentWithWallet):
    def __init__(
        self,
        proposal_wallet_name: str,
        proposal_wallet_pass: str,
        termination_wallet_name: str,
        termination_wallet_pass: str,
        market_name: str,
        market_code: str,
        asset_name: str,
        asset_dp: int,
        proposal_key_name: Optional[str] = None,
        termination_key_name: Optional[str] = None,
        market_config: Optional[MarketConfig] = None,
        tag: Optional[str] = None,
        settlement_price: Optional[float] = None,
        initial_mint: Optional[float] = 1e9,
    ):

        self.wallet_name = proposal_wallet_name
        self.wallet_pass = proposal_wallet_pass
        self.key_name = proposal_key_name

        self.termination_wallet_name = termination_wallet_name
        self.termination_wallet_pass = termination_wallet_pass
        self.termination_key_name = termination_key_name

        self.market_name = market_name
        self.market_code = market_code

        self.asset_dp = asset_dp
        self.asset_name = asset_name

        self.initial_mint = initial_mint

        self.market_config = (
            market_config if market_config is not None else MarketConfig()
        )

        self.tag = tag
        self.settlement_price = settlement_price

    def initialise(
        self,
        vega: Union[VegaServiceNull, VegaServiceNetwork],
        create_wallet: bool = True,
        mint_wallet: bool = True,
    ):

        super().initialise(vega=vega, create_wallet=create_wallet)
        self.vega.create_wallet(
            self.termination_wallet_name,
            self.termination_wallet_pass,
            self.termination_key_name,
        )

        self.vega.wait_for_total_catchup()
        if mint_wallet:
            self.vega.mint(
                self.wallet_name, asset="VOTE", amount=1e4, key_name=self.key_name
            )

        self.vega.wait_for_total_catchup()

        if self.vega.find_asset_id(symbol=self.asset_name) is None:
            self.vega.create_asset(
                self.wallet_name,
                name=self.asset_name,
                symbol=self.asset_name,
                decimals=self.asset_dp,
                max_faucet_amount=5e10,
                key_name=self.key_name,
            )

        self.vega.wait_for_total_catchup()
        self.asset_id = self.vega.find_asset_id(symbol=self.asset_name)

        self.vega.wait_for_total_catchup()
        if mint_wallet:
            self.vega.mint(
                self.wallet_name,
                asset=self.asset_id,
                amount=self.initial_mint,
                key_name=self.key_name,
            )

        if self.vega.find_market_id(name=self.market_name) is None:
            # Add market information and asset information to market config
            self.market_config.set("instrument.name", self.market_name)
            self.market_config.set("instrument.code", self.market_code)
            self.market_config.set("instrument.future.settlement_asset", self.asset_id)
            self.market_config.set("instrument.future.quote_name", self.asset_name)
            self.market_config.set(
                "instrument.future.settlement_data_decimals", self.asset_dp
            )
            self.market_config.set(
                "instrument.future.terminating_key",
                self.vega.wallet.public_key(
                    self.termination_wallet_name, self.termination_key_name
                ),
            )

            self.vega.wait_for_total_catchup()
            self.vega.create_market_from_config(
                proposal_wallet_name=self.wallet_name,
                proposal_key_name=self.key_name,
                market_config=self.market_config,
            )

        self.vega.wait_for_total_catchup()
        self.market_id = self.vega.find_market_id(name=self.market_name)

    def finalise(self):
        if self.settlement_price is not None:
            self.vega.settle_market(
                self.termination_wallet_name,
                self.settlement_price,
                self.market_id,
                self.termination_key_name,
            )
            self.vega.wait_for_total_catchup()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vega_sim',
 'vega_sim.api',
 'vega_sim.environment',
 'vega_sim.grpc',
 'vega_sim.parameter_test',
 'vega_sim.parameter_test.parameter',
 'vega_sim.proto',
 'vega_sim.proto.blockexplorer',
 'vega_sim.proto.data_node',
 'vega_sim.proto.data_node.api',
 'vega_sim.proto.data_node.api.v1',
 'vega_sim.proto.data_node.api.v2',
 'vega_sim.proto.vega',
 'vega_sim.proto.vega.api',
 'vega_sim.proto.vega.api.v1',
 'vega_sim.proto.vega.checkpoint',
 'vega_sim.proto.vega.checkpoint.v1',
 'vega_sim.proto.vega.commands',
 'vega_sim.proto.vega.commands.v1',
 'vega_sim.proto.vega.data',
 'vega_sim.proto.vega.data.v1',
 'vega_sim.proto.vega.events',
 'vega_sim.proto.vega.events.v1',
 'vega_sim.proto.vega.snapshot',
 'vega_sim.proto.vega.snapshot.v1',
 'vega_sim.proto.vega.wallet',
 'vega_sim.proto.vega.wallet.v1',
 'vega_sim.reinforcement',
 'vega_sim.reinforcement.agents',
 'vega_sim.reinforcement.full_market_sim',
 'vega_sim.reinforcement.full_market_sim.utils',
 'vega_sim.replay',
 'vega_sim.scenario',
 'vega_sim.scenario.common',
 'vega_sim.scenario.common.utils',
 'vega_sim.scenario.comprehensive_market',
 'vega_sim.scenario.configurable_market',
 'vega_sim.scenario.curve_market_maker',
 'vega_sim.scenario.ideal_market_maker',
 'vega_sim.scenario.ideal_market_maker.utils',
 'vega_sim.scenario.ideal_market_maker_v2',
 'vega_sim.scenario.ideal_market_maker_v2.utils',
 'vega_sim.scenario.market_crash',
 'vega_sim.scenario.multi_market',
 'vega_sim.wallet']

package_data = \
{'': ['*'],
 'vega_sim': ['vegahome/*',
              'vegahome/config/data-node/*',
              'vegahome/config/faucet/*',
              'vegahome/config/node/*',
              'vegahome/config/wallet-service/networks/*',
              'vegahome/data/*',
              'vegahome/data/faucet/wallets/*',
              'vegahome/data/node/wallets/ethereum/*',
              'vegahome/data/node/wallets/vega/*',
              'vegahome/data/wallet-service/rsa-keys/*',
              'vegahome/data/wallets/*'],
 'vega_sim.reinforcement': ['theory_intro/*', 'theory_intro/images/*']}

install_requires = \
['PyNaCl>=1.5.0,<2.0.0',
 'grpc-gateway-protoc-gen-openapiv2>=0.1.0,<0.2.0',
 'grpcio-tools==1.48.1',
 'inflection>=0.5.1,<0.6.0',
 'numpy>=1.22.4,<2.0.0',
 'pandas>=1.4.2,<2.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'requests>=2.28.0,<3.0.0',
 'scipy>=1.8.1,<2.0.0',
 'toml>=0.10.2,<0.11.0']

extras_require = \
{'agents': ['TA-Lib>=0.4.25,<0.5.0'],
 'jupyter': ['jupyterlab>=3.4.3,<4.0.0',
             'jupyter>=1.0.0,<2.0.0',
             'matplotlib>=3.5.2,<4.0.0',
             'ipywidgets>=7.7.1,<8.0.0',
             'plotly>=5.10.0,<6.0.0'],
 'learning': ['torch>=1.11.0,<2.0.0',
              'tqdm>=4.64.0,<5.0.0',
              'matplotlib>=3.5.2,<4.0.0'],
 'profile': ['snakeviz>=2.1.1,<3.0.0', 'pytest-profiling>=1.7.0,<2.0.0']}

setup_kwargs = {
    'name': 'vega-sim',
    'version': '0.62.1',
    'description': 'Simulator for running self-contained Vega chain on local PC',
    'long_description': 'None',
    'author': 'Tom McLean',
    'author_email': 'tom@vegaprotocol.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.9,<3.11',
}


setup(**setup_kwargs)



from typing import List, Callable, Any
import torch
import torch.nn as nn
import numpy as np


def apply_funcs(value, funcs: List[Callable[..., Any]]):
    """
    composition of functions applied to value
    """
    res = value
    for f in funcs:
        res = f(res)
    return res


def to_torch(arr: np.array, dtype: torch.dtype = torch.float, device: str = "cpu"):
    return torch.tensor(arr, dtype=dtype, device=device)


def toggle(m: nn.Module, to: bool):
    """
    freezes or unfreezes a model's parameters
    """
    for p in m.parameters():
        p.requires_grad_(to)


def set_seed(seed):
    torch.manual_seed(seed)
    np.random.seed(seed)



from dataclasses import dataclass
import numpy as np
from collections import namedtuple, defaultdict
from typing import List, Tuple
import os
from functools import partial
from tqdm import tqdm

import torch
import torch.nn as nn
from torch.distributions.categorical import Categorical
from vega_sim.reinforcement.agents.learning_agent import AbstractAction, LearningAgent

import pickle

from vega_sim.reinforcement.networks import (
    Softmax,
    FFN,
    FFN_Params_Normal,
    FFN_fix_fol_Q,
)
from vega_sim.reinforcement.helpers import apply_funcs, to_torch, toggle
from vega_sim.reinforcement.la_market_state import LAMarketState

from vega_sim.reinforcement.distributions import (
    lognorm_sample,
    lognorm_logprob,
    reg_policy,
)

from vega_sim.api.helpers import num_from_padded_int

from vega_sim.environment import VegaState
from vega_sim.environment.agent import StateAgentWithWallet
from vega_sim.null_service import VegaServiceNull
from vega_sim.proto.vega import markets as markets_protos

WalletConfig = namedtuple("WalletConfig", ["name", "passphrase"])

# Send selling/buying MOs to hit LP orders
WALLET = WalletConfig("learner", "learner")


@dataclass
class Action:
    buy: bool
    sell: bool


class LearningAgentFixedVol(LearningAgent):
    def __init__(
        self,
        device: str,
        logfile_pol_imp: str,
        logfile_pol_eval: str,
        logfile_pnl: str,
        discount_factor: float,
        num_levels: int,
        wallet_name: str,
        wallet_pass: str,
        market_name: str,
        initial_balance: int,
        position_decimals: int,
        inventory_penalty: float = 1.0,
    ):
        super().__init__(
            device=device,
            logfile_pol_imp=logfile_pol_imp,
            logfile_pol_eval=logfile_pol_eval,
            logfile_pnl=logfile_pnl,
            discount_factor=discount_factor,
            num_levels=num_levels,
            wallet_name=wallet_name,
            wallet_pass=wallet_pass,
            market_name=market_name,
            initial_balance=initial_balance,
            position_decimals=position_decimals,
            inventory_penalty=inventory_penalty,
        )
        self.volume = 10 ** (-self.position_decimals)

        # Dimensions of state and action
        self.num_levels = num_levels
        self.state_dim = 6 + 4 * self.num_levels  # from MarketState
        action_discrete_dim = 3

        # NN for Q-fun and its optimizer
        self.q_func = FFN_fix_fol_Q(state_dim=self.state_dim)
        self.optimizer_q = torch.optim.RMSprop(self.q_func.parameters(), lr=0.01)

        # NN for policy and its optimizer
        self.policy_discr = FFN(
            sizes=[self.state_dim, 4096, action_discrete_dim],
            activation=nn.ReLU,
            output_activation=Softmax,
        )  # this network decides whether to buy/sell/do nothing
        self.optimizer_pol = torch.optim.RMSprop(
            list(self.policy_discr.parameters()), lr=0.001
        )

    def move_to_device(self):
        self.q_func.to(self.device)
        self.policy_discr.to(self.device)

    def move_to_cpu(self):
        self.q_func.to("cpu")
        self.policy_discr.to("cpu")

    def _update_memory(
        self,
        state: LAMarketState,
        action: Action,
        reward: float,
        next_state: LAMarketState,
    ):
        """
        Ensure what is the input?
        """
        self.memory["state"].append(state.to_array())
        # action_discrete = 0 if sell
        # action_discrete = 1 if buy
        # action_discrete = 2 if they do not do anything
        if action.sell:
            self.memory["action_discrete"].append([0])
        elif action.buy:
            self.memory["action_discrete"].append([1])
        else:
            self.memory["action_discrete"].append([2])

        self.memory["reward"].append([reward])
        if next_state is not np.nan:
            self.memory["next_state"].append(next_state.to_array())
        else:
            self.memory["next_state"].append(np.nan * np.ones_like(state.to_array()))

        return 0

    def create_dataloader(self, batch_size):
        """
        creates dataset and dataloader for training.
        """
        to_torch_device = partial(to_torch, device=self.device)
        dataset_state = apply_funcs(
            value=self.memory["state"], funcs=(np.stack, to_torch_device)
        )
        dataset_action_discrete = apply_funcs(
            value=self.memory["action_discrete"],
            funcs=(np.array, partial(to_torch, dtype=torch.int64, device=self.device)),
        )
        dataset_reward = apply_funcs(
            value=self.memory["reward"], funcs=(np.array, to_torch_device)
        )
        dataset_next_state = apply_funcs(
            value=self.memory["next_state"], funcs=(np.stack, to_torch_device)
        )

        dataset = torch.utils.data.TensorDataset(
            *(
                dataset_state,
                dataset_action_discrete,
                dataset_reward,
                dataset_next_state,
            )
        )
        dataloader = torch.utils.data.DataLoader(
            dataset, batch_size=batch_size, shuffle=True
        )
        return dataloader

    def empty_action(self) -> AbstractAction:
        return Action(False, False)

    def step(self, vega_state: VegaState):
        learning_state = self.state(self.vega)
        self.step_num += 1
        self.latest_action = self._step(learning_state)
        self.latest_state = learning_state

        if learning_state.full_balance <= 0:
            return
        if learning_state.market_in_auction:
            return

        if self.latest_action.buy or self.latest_action.sell:
            try:
                self.vega.submit_market_order(
                    trading_wallet=self.wallet_name,
                    market_id=self.market_id,
                    side="SIDE_BUY" if self.latest_action.buy else "SIDE_SELL",
                    volume=self.volume,
                    wait=False,
                    fill_or_kill=False,
                )
            except Exception as e:
                print(e)

    def _step(self, vega_state: LAMarketState) -> Action:
        # learned policy
        state = vega_state.to_array().reshape(1, -1)  # adding batch_dimension
        state = torch.from_numpy(state).float()  # .to(self.device)

        with torch.no_grad():
            c = self.sample_action(state=state, sim=True)
        choice = int(c.item())

        return Action(buy=choice == 0, sell=choice == 1)

    def _step_heuristic(self, vega_state: LAMarketState) -> Action:
        if (
            vega_state.position <= 0
            and vega_state.ask_prices[0] < vega_state.next_price
        ):
            choice = 0  # buy
        elif (
            vega_state.position >= 0
            and vega_state.bid_prices[0] > vega_state.next_price
        ):
            choice = 1  # sell
        else:
            choice = 2  # do nothing

        return Action(buy=choice == 0, sell=choice == 1)

    def sample_action(
        self,
        state: torch.Tensor,
        sim: bool = True,
        evaluate: bool = False,
    ):
        """
        Sample an action.

        Returns
        -------
        c is a tensor of shape (batch_size, 1) returning the sampled action from {sell, buy, do nothing} (i.e. c is filled with values from {0,1,2})
        """

        probs = self.policy_discr(state)
        if sim:
            m = Categorical(probs)
            c = m.sample()
        else:
            c = probs
        if evaluate:
            c = torch.max(probs, 1, keepdim=True)[1]
        return c

    def policy_eval(
        self,
        batch_size: int,
        n_epochs: int,
    ):
        toggle(self.policy_discr, to=False)
        toggle(self.q_func, to=True)

        dataloader = self.create_dataloader(batch_size=batch_size)

        pbar = tqdm(total=n_epochs)
        for epoch in range(n_epochs):
            for (
                i,
                (
                    batch_state,
                    batch_action_discrete,
                    batch_reward,
                    batch_next_state,
                ),
            ) in enumerate(dataloader):
                next_state_terminal = torch.isnan(
                    batch_next_state
                ).float()  # shape (batch_size, dim_state)
                batch_next_state[next_state_terminal.eq(True)] = batch_state[
                    next_state_terminal.eq(True)
                ]
                self.optimizer_q.zero_grad()

                pred = torch.gather(
                    self.q_func(batch_state),
                    dim=1,
                    index=batch_action_discrete,
                )

                with torch.no_grad():
                    v = self.v_func(batch_next_state)
                    target = (
                        batch_reward
                        + (1 - next_state_terminal.float().mean(1, keepdim=True))
                        * self.discount_factor
                        * v
                    )
                loss = torch.pow(pred - target, 2).mean()
                loss.backward()
                self.optimizer_q.step()
            self.losses["q"].append(loss.item())
            # logging loss
            with open(self.logfile_pol_eval, "a") as f:
                f.write(
                    "{},{:.2e},{:.2e},{:.2e}\n".format(
                        epoch + self.lerningIteration * n_epochs,
                        loss.item(),
                        self.coefH_discr,
                        self.coefH_cont,
                    )
                )
            pbar.update(1)
        return 0

    def v_func(self, state):
        """
        v(x) = E[q(x,A)] = E[q(x,A)|C=0]p(C=0) + E[q(x,A)|C=1]p(C=1) + E[q(x)|C=2]p(C=2)

        if C==0 --> sell,
        if C==1 --> buy,
        if C==2 --> does nothing,
        Parameters
        ----------
        state: torch.Tensor
            State. Tensor of shape (batch_size,
        n_mc: torch.Tensor
            Number of Monte Carlo samples to calculate
        """

        q = self.q_func(state)
        c = self.sample_action(
            state, sim=False
        )  # .unravel().reshape((state.shape[0], 1))

        v = c[:, 0] * (q[:, 0] - self.coefH_discr * torch.log(c[:, 0]))
        v += c[:, 1] * (q[:, 1] - self.coefH_discr * torch.log(c[:, 1]))
        v += c[:, 2] * (q[:, 2] - self.coefH_discr * torch.log(c[:, 2]))
        self.q_func.train()
        return v

    def D_KL(self, state):
        """
        KL divergence between pi(.|x) and the unnormalised density exp(Q(x,.)),
        where pi(.|x) is a LogNormal distribution
        """
        # c = self.sample_action(state).reshape((state.shape[0], 1))

        q = self.q_func(state)
        c = self.sample_action(
            state, sim=False
        )  # .unravel().reshape((state.shape[0], 1))

        d_kl = c[:, 0] * (self.coefH_discr * torch.log(c[:, 0]) - q[:, 0])
        d_kl += c[:, 1] * (self.coefH_discr * torch.log(c[:, 1]) - q[:, 1])
        d_kl += c[:, 2] * (self.coefH_discr * torch.log(c[:, 2]) - q[:, 2])

        return d_kl

    def policy_improvement(self, batch_size: int, n_epochs: int):
        toggle(self.policy_discr, to=True)
        toggle(self.q_func, to=False)

        dataloader = self.create_dataloader(batch_size=batch_size)

        pbar = tqdm(total=n_epochs)
        for epoch in range(n_epochs):
            for i, (batch_state, _, _, _) in enumerate(dataloader):
                self.optimizer_pol.zero_grad()
                d_kl = self.D_KL(batch_state).mean()
                d_kl.backward()
                # nn.utils.clip_grad_norm_(self.policy_volume.parameters(), max_norm=1.)
                self.optimizer_pol.step()
            self.losses["d_kl"].append(d_kl.item())
            with open(self.logfile_pol_imp, "a") as f:
                f.write(
                    "{},{:.4f}\n".format(
                        epoch + n_epochs * self.lerningIteration, d_kl.item()
                    )
                )
            pbar.update(1)

        # update the coefficients for the next run
        self.coefH_discr = max(self.coefH_discr * 0.99, 1e-10)
        self.coefH_cont = max(self.coefH_cont * 0.99, 1e-10)
        self.lerningIteration += 1
        return 0

    def save(self, results_dir: str):
        filename = os.path.join(results_dir, "agent.pth.tar")
        d = {
            "losses": self.losses,
            "q": self.q_func.state_dict(),
            "policy_discr": self.policy_discr.state_dict(),
            "iteration": self.lerningIteration,
            "coefH_discr": self.coefH_discr,
        }
        torch.save(d, filename)

        filename_for_memory = os.path.join(results_dir, "memory.pickle")
        with open(filename_for_memory, "wb") as f:
            pickle.dump(self.memory, f)

    def load(self, results_dir: str):
        filename = os.path.join(results_dir, "agent.pth.tar")
        d = torch.load(filename, map_location="cpu")
        self.q_func.load_state_dict(d["q"])
        self.policy_discr.load_state_dict(d["policy_discr"])

        self.lerningIteration = d["iteration"]
        self.coefH_discr = d["coefH_discr"]

        filename_for_memory = os.path.join(results_dir, "memory.pickle")
        with open(filename_for_memory, "rb") as f:
            memory = pickle.load(f)
        self.memory = memory



import numpy as np


def GBM_model(T=0.2, mu=0.05, sigma=0.2, Midprice=100000):
    """
    Given the Midprice at time 0, calculate the asset price based on Geometric Brownian motion Model
    """
    Termination = int(T * 3600)
    t = np.linspace(0, Termination, Termination + 1).astype(int)
    dW = np.random.randn(len(t) - 1)
    W = np.cumsum(dW)
    W = np.insert(W, 0, 0)

    S = Midprice * np.exp((mu - 0.5 * sigma**2) * t + sigma * W)

    return t, S


def RW_model(
    random_state=np.random.RandomState(seed=1),
    T=3 / 24 / 365.25,
    dt=1 / 60 / 24 / 365.25,
    mdp=5,
    sigma=1,
    Midprice=0.3,
):
    """
    Given the Midprice at time 0, simulate the asset price based on Random Walk Model
    """

    Termination = int(T / dt)
    time_step = np.linspace(0, Termination, Termination + 1).astype(int)
    # use seed to make sure gernerate same random number (use to compare different strategy)
    # random_state = np.random.RandomState()

    dW = np.sqrt(dt) * random_state.randn(len(time_step) - 1)
    S = np.zeros(len(time_step))
    S[0] = Midprice
    # for i in range(len(time_step)-1):
    #     S[i+1] = S[i] + sigma*dW[i]

    for i in range(len(time_step) - 1):
        S[i + 1] = S[i] * np.exp(-0.5 * sigma * sigma * dt + sigma * dW[i])

    # market decimal place
    S = np.round(S, mdp)

    # make sure the simulated external price is larger than 0
    while True:
        random_state = np.random.RandomState()
        # use seed to make sure gernerate same random number (use to compare different strategy)
        # random_state = np.random.RandomState(seed=123)
        dW = np.sqrt(dt) * random_state.randn(len(time_step) - 1)
        # Simulate external midprice
        for i in range(len(time_step) - 1):
            S[i + 1] = S[i] + sigma * dW[i]
        # market decimal place
        S = np.round(S, mdp)

        if (S > 0).all():
            break
    return time_step, S



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vega/assets.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x11vega/assets.proto\x12\x04vega"\xed\x01\n\x05\x41sset\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12,\n\x07\x64\x65tails\x18\x02 \x01(\x0b\x32\x12.vega.AssetDetailsR\x07\x64\x65tails\x12*\n\x06status\x18\x03 \x01(\x0e\x32\x12.vega.Asset.StatusR\x06status"z\n\x06Status\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x13\n\x0fSTATUS_PROPOSED\x10\x01\x12\x13\n\x0fSTATUS_REJECTED\x10\x02\x12\x1a\n\x16STATUS_PENDING_LISTING\x10\x03\x12\x12\n\x0eSTATUS_ENABLED\x10\x04"\xe0\x01\n\x0c\x41ssetDetails\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x16\n\x06symbol\x18\x02 \x01(\tR\x06symbol\x12\x1a\n\x08\x64\x65\x63imals\x18\x04 \x01(\x04R\x08\x64\x65\x63imals\x12\x18\n\x07quantum\x18\x05 \x01(\tR\x07quantum\x12\x39\n\rbuiltin_asset\x18\x65 \x01(\x0b\x32\x12.vega.BuiltinAssetH\x00R\x0c\x62uiltinAsset\x12#\n\x05\x65rc20\x18\x66 \x01(\x0b\x32\x0b.vega.ERC20H\x00R\x05\x65rc20B\x08\n\x06sourceJ\x04\x08\x03\x10\x04"C\n\x0c\x42uiltinAsset\x12\x33\n\x16max_faucet_amount_mint\x18\x01 \x01(\tR\x13maxFaucetAmountMint"\x88\x01\n\x05\x45RC20\x12)\n\x10\x63ontract_address\x18\x01 \x01(\tR\x0f\x63ontractAddress\x12%\n\x0elifetime_limit\x18\x02 \x01(\tR\rlifetimeLimit\x12-\n\x12withdraw_threshold\x18\x03 \x01(\tR\x11withdrawThreshold"{\n\x12\x41ssetDetailsUpdate\x12\x18\n\x07quantum\x18\x05 \x01(\tR\x07quantum\x12)\n\x05\x65rc20\x18\x65 \x01(\x0b\x32\x11.vega.ERC20UpdateH\x00R\x05\x65rc20B\x08\n\x06sourceJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05"c\n\x0b\x45RC20Update\x12%\n\x0elifetime_limit\x18\x01 \x01(\tR\rlifetimeLimit\x12-\n\x12withdraw_threshold\x18\x02 \x01(\tR\x11withdrawThresholdB\'Z%code.vegaprotocol.io/vega/protos/vegab\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "vega.assets_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"Z%code.vegaprotocol.io/vega/protos/vega"
    _ASSET._serialized_start = 28
    _ASSET._serialized_end = 265
    _ASSET_STATUS._serialized_start = 143
    _ASSET_STATUS._serialized_end = 265
    _ASSETDETAILS._serialized_start = 268
    _ASSETDETAILS._serialized_end = 492
    _BUILTINASSET._serialized_start = 494
    _BUILTINASSET._serialized_end = 561
    _ERC20._serialized_start = 564
    _ERC20._serialized_end = 700
    _ASSETDETAILSUPDATE._serialized_start = 702
    _ASSETDETAILSUPDATE._serialized_end = 825
    _ERC20UPDATE._serialized_start = 827
    _ERC20UPDATE._serialized_end = 926
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from ...api.v1 import corestate_pb2 as vega_dot_api_dot_v1_dot_corestate__pb2


class CoreStateServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListAccounts = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListAccounts",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListAccountsRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListAccountsResponse.FromString,
        )
        self.ListAssets = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListAssets",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListAssetsRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListAssetsResponse.FromString,
        )
        self.ListNetworkParameters = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListNetworkParameters",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkParametersRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkParametersResponse.FromString,
        )
        self.ListNetworkLimits = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListNetworkLimits",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkLimitsRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkLimitsResponse.FromString,
        )
        self.ListParties = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListParties",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesResponse.FromString,
        )
        self.ListValidators = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListValidators",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListValidatorsRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListValidatorsResponse.FromString,
        )
        self.ListMarkets = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListMarkets",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsResponse.FromString,
        )
        self.ListProposals = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListProposals",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListProposalsRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListProposalsResponse.FromString,
        )
        self.ListMarketsData = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListMarketsData",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsDataRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsDataResponse.FromString,
        )
        self.ListVotes = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListVotes",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListVotesRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListVotesResponse.FromString,
        )
        self.ListPartiesStake = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListPartiesStake",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesStakeRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesStakeResponse.FromString,
        )
        self.ListDelegations = channel.unary_unary(
            "/vega.api.v1.CoreStateService/ListDelegations",
            request_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListDelegationsRequest.SerializeToString,
            response_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListDelegationsResponse.FromString,
        )


class CoreStateServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ListAccounts(self, request, context):
        """Accounts list

        Return a list of accounts
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListAssets(self, request, context):
        """Assets list

        Return a list of assets
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListNetworkParameters(self, request, context):
        """Network parameters list

        Return a list of network paramters
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListNetworkLimits(self, request, context):
        """Network limits list

        Return a list of network limits
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListParties(self, request, context):
        """Parties list

        Return a list of parties
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListValidators(self, request, context):
        """Validators list

        Return a list of validators
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListMarkets(self, request, context):
        """Markets list

        Return a list of markets
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListProposals(self, request, context):
        """Proposals list

        Return a list of proposals
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListMarketsData(self, request, context):
        """Markets data list

        Return a list of markets data
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListVotes(self, request, context):
        """Votes list

        Return a list of votes
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListPartiesStake(self, request, context):
        """Parties stake list

        Return a list of parties stake
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListDelegations(self, request, context):
        """Delegations list

        Return a list of delegations
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_CoreStateServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "ListAccounts": grpc.unary_unary_rpc_method_handler(
            servicer.ListAccounts,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListAccountsRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListAccountsResponse.SerializeToString,
        ),
        "ListAssets": grpc.unary_unary_rpc_method_handler(
            servicer.ListAssets,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListAssetsRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListAssetsResponse.SerializeToString,
        ),
        "ListNetworkParameters": grpc.unary_unary_rpc_method_handler(
            servicer.ListNetworkParameters,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkParametersRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkParametersResponse.SerializeToString,
        ),
        "ListNetworkLimits": grpc.unary_unary_rpc_method_handler(
            servicer.ListNetworkLimits,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkLimitsRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkLimitsResponse.SerializeToString,
        ),
        "ListParties": grpc.unary_unary_rpc_method_handler(
            servicer.ListParties,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesResponse.SerializeToString,
        ),
        "ListValidators": grpc.unary_unary_rpc_method_handler(
            servicer.ListValidators,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListValidatorsRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListValidatorsResponse.SerializeToString,
        ),
        "ListMarkets": grpc.unary_unary_rpc_method_handler(
            servicer.ListMarkets,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsResponse.SerializeToString,
        ),
        "ListProposals": grpc.unary_unary_rpc_method_handler(
            servicer.ListProposals,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListProposalsRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListProposalsResponse.SerializeToString,
        ),
        "ListMarketsData": grpc.unary_unary_rpc_method_handler(
            servicer.ListMarketsData,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsDataRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsDataResponse.SerializeToString,
        ),
        "ListVotes": grpc.unary_unary_rpc_method_handler(
            servicer.ListVotes,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListVotesRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListVotesResponse.SerializeToString,
        ),
        "ListPartiesStake": grpc.unary_unary_rpc_method_handler(
            servicer.ListPartiesStake,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesStakeRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesStakeResponse.SerializeToString,
        ),
        "ListDelegations": grpc.unary_unary_rpc_method_handler(
            servicer.ListDelegations,
            request_deserializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListDelegationsRequest.FromString,
            response_serializer=vega_dot_api_dot_v1_dot_corestate__pb2.ListDelegationsResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "vega.api.v1.CoreStateService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class CoreStateService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ListAccounts(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListAccounts",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListAccountsRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListAccountsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListAssets(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListAssets",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListAssetsRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListAssetsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListNetworkParameters(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListNetworkParameters",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkParametersRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkParametersResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListNetworkLimits(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListNetworkLimits",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkLimitsRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListNetworkLimitsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListParties(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListParties",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListValidators(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListValidators",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListValidatorsRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListValidatorsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListMarkets(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListMarkets",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListProposals(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListProposals",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListProposalsRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListProposalsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListMarketsData(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListMarketsData",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsDataRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListMarketsDataResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListVotes(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListVotes",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListVotesRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListVotesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListPartiesStake(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListPartiesStake",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesStakeRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListPartiesStakeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListDelegations(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/vega.api.v1.CoreStateService/ListDelegations",
            vega_dot_api_dot_v1_dot_corestate__pb2.ListDelegationsRequest.SerializeToString,
            vega_dot_api_dot_v1_dot_corestate__pb2.ListDelegationsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from collections import namedtuple

WalletConfig = namedtuple("WalletConfig", ["name", "passphrase"])

# Set up wallets for parties proposing market
MM_WALLET = WalletConfig("mm", "pin")

# Set up wallets for parties to force auction pass
AUCTION1_WALLET = WalletConfig("AUCTION1", "AUCTION1pass")
AUCTION2_WALLET = WalletConfig("AUCTION2", "AUCTION2pass")

# Set up the wallet for market termination
TERMINATE_WALLET = WalletConfig("FJMKnwfZdd48C8NqvYrG", "bY3DxwtsCstMIIZdNpKs")

# informed trader wallet
INFORMED_WALLET = WalletConfig("INFORMED", "INFORMEDpass")


def create_agent_wallets(n: int = 1, prefix: str = "agent"):
    """Function creates a list of n WalletConfig tuples for use in a scenario"""
    return [WalletConfig(prefix + str(i), "pass") for i in range(n)]



from vega_sim.reinforcement.agents.learning_agent_MO_with_vol import (
    LearningAgentWithVol,
)

from vega_sim.scenario.ideal_market_maker.environments import MarketEnvironment
from vega_sim.service import VegaService


class RLMarketEnvironment(MarketEnvironment):
    def add_learning_agent(self, agent: LearningAgentWithVol):
        self.learning_agent = agent
        self.agents = self._base_agents + [agent]

    def step(self, vega: VegaService):
        super().step(vega=vega)

        state = self.state_func(vega)
        # Learning agent
        self.learning_agent.step(state, random=False)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vega/commands/v1/transaction.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ...commands.v1 import commands_pb2 as vega_dot_commands_dot_v1_dot_commands__pb2
from ...commands.v1 import (
    validator_commands_pb2 as vega_dot_commands_dot_v1_dot_validator__commands__pb2,
)
from ...commands.v1 import data_pb2 as vega_dot_commands_dot_v1_dot_data__pb2
from ...commands.v1 import signature_pb2 as vega_dot_commands_dot_v1_dot_signature__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n"vega/commands/v1/transaction.proto\x12\x10vega.commands.v1\x1a\x1fvega/commands/v1/commands.proto\x1a)vega/commands/v1/validator_commands.proto\x1a\x1bvega/commands/v1/data.proto\x1a vega/commands/v1/signature.proto"\xaa\x12\n\tInputData\x12\x14\n\x05nonce\x18\x01 \x01(\x04R\x05nonce\x12!\n\x0c\x62lock_height\x18\x02 \x01(\x04R\x0b\x62lockHeight\x12O\n\x10order_submission\x18\xe9\x07 \x01(\x0b\x32!.vega.commands.v1.OrderSubmissionH\x00R\x0forderSubmission\x12U\n\x12order_cancellation\x18\xea\x07 \x01(\x0b\x32#.vega.commands.v1.OrderCancellationH\x00R\x11orderCancellation\x12L\n\x0forder_amendment\x18\xeb\x07 \x01(\x0b\x32 .vega.commands.v1.OrderAmendmentH\x00R\x0eorderAmendment\x12X\n\x13withdraw_submission\x18\xec\x07 \x01(\x0b\x32$.vega.commands.v1.WithdrawSubmissionH\x00R\x12withdrawSubmission\x12X\n\x13proposal_submission\x18\xed\x07 \x01(\x0b\x32$.vega.commands.v1.ProposalSubmissionH\x00R\x12proposalSubmission\x12L\n\x0fvote_submission\x18\xee\x07 \x01(\x0b\x32 .vega.commands.v1.VoteSubmissionH\x00R\x0evoteSubmission\x12w\n\x1eliquidity_provision_submission\x18\xef\x07 \x01(\x0b\x32..vega.commands.v1.LiquidityProvisionSubmissionH\x00R\x1cliquidityProvisionSubmission\x12X\n\x13\x64\x65legate_submission\x18\xf0\x07 \x01(\x0b\x32$.vega.commands.v1.DelegateSubmissionH\x00R\x12\x64\x65legateSubmission\x12^\n\x15undelegate_submission\x18\xf1\x07 \x01(\x0b\x32&.vega.commands.v1.UndelegateSubmissionH\x00R\x14undelegateSubmission\x12}\n liquidity_provision_cancellation\x18\xf2\x07 \x01(\x0b\x32\x30.vega.commands.v1.LiquidityProvisionCancellationH\x00R\x1eliquidityProvisionCancellation\x12t\n\x1dliquidity_provision_amendment\x18\xf3\x07 \x01(\x0b\x32-.vega.commands.v1.LiquidityProvisionAmendmentH\x00R\x1bliquidityProvisionAmendment\x12\x39\n\x08transfer\x18\xf4\x07 \x01(\x0b\x32\x1a.vega.commands.v1.TransferH\x00R\x08transfer\x12L\n\x0f\x63\x61ncel_transfer\x18\xf5\x07 \x01(\x0b\x32 .vega.commands.v1.CancelTransferH\x00R\x0e\x63\x61ncelTransfer\x12\x46\n\rannounce_node\x18\xf6\x07 \x01(\x0b\x32\x1e.vega.commands.v1.AnnounceNodeH\x00R\x0c\x61nnounceNode\x12h\n\x19\x62\x61tch_market_instructions\x18\xf7\x07 \x01(\x0b\x32).vega.commands.v1.BatchMarketInstructionsH\x00R\x17\x62\x61tchMarketInstructions\x12:\n\tnode_vote\x18\xd2\x0f \x01(\x0b\x32\x1a.vega.commands.v1.NodeVoteH\x00R\x08nodeVote\x12I\n\x0enode_signature\x18\xd3\x0f \x01(\x0b\x32\x1f.vega.commands.v1.NodeSignatureH\x00R\rnodeSignature\x12@\n\x0b\x63hain_event\x18\xd4\x0f \x01(\x0b\x32\x1c.vega.commands.v1.ChainEventH\x00R\nchainEvent\x12\\\n\x15key_rotate_submission\x18\xd5\x0f \x01(\x0b\x32%.vega.commands.v1.KeyRotateSubmissionH\x00R\x13keyRotateSubmission\x12\x62\n\x17state_variable_proposal\x18\xd6\x0f \x01(\x0b\x32\'.vega.commands.v1.StateVariableProposalH\x00R\x15stateVariableProposal\x12X\n\x13validator_heartbeat\x18\xd7\x0f \x01(\x0b\x32$.vega.commands.v1.ValidatorHeartbeatH\x00R\x12validatorHeartbeat\x12u\n\x1e\x65thereum_key_rotate_submission\x18\xd8\x0f \x01(\x0b\x32-.vega.commands.v1.EthereumKeyRotateSubmissionH\x00R\x1b\x65thereumKeyRotateSubmission\x12h\n\x19protocol_upgrade_proposal\x18\xd9\x0f \x01(\x0b\x32).vega.commands.v1.ProtocolUpgradeProposalH\x00R\x17protocolUpgradeProposal\x12O\n\x10issue_signatures\x18\xda\x0f \x01(\x0b\x32!.vega.commands.v1.IssueSignaturesH\x00R\x0fissueSignatures\x12_\n\x16oracle_data_submission\x18\xb9\x17 \x01(\x0b\x32&.vega.commands.v1.OracleDataSubmissionH\x00R\x14oracleDataSubmissionB\t\n\x07\x63ommandJ\x06\x08\xa1\x1f\x10\xa2\x1f"\x92\x02\n\x0bTransaction\x12\x1d\n\ninput_data\x18\x01 \x01(\x0cR\tinputData\x12\x39\n\tsignature\x18\x02 \x01(\x0b\x32\x1b.vega.commands.v1.SignatureR\tsignature\x12\x1b\n\x07\x61\x64\x64ress\x18\xe9\x07 \x01(\tH\x00R\x07\x61\x64\x64ress\x12\x1a\n\x07pub_key\x18\xea\x07 \x01(\tH\x00R\x06pubKey\x12\x36\n\x07version\x18\xd0\x0f \x01(\x0e\x32\x1b.vega.commands.v1.TxVersionR\x07version\x12\x30\n\x03pow\x18\xb8\x17 \x01(\x0b\x32\x1d.vega.commands.v1.ProofOfWorkR\x03powB\x06\n\x04\x66rom"5\n\x0bProofOfWork\x12\x10\n\x03tid\x18\x01 \x01(\tR\x03tid\x12\x14\n\x05nonce\x18\x02 \x01(\x04R\x05nonce*S\n\tTxVersion\x12\x1a\n\x16TX_VERSION_UNSPECIFIED\x10\x00\x12\x11\n\rTX_VERSION_V2\x10\x02\x12\x11\n\rTX_VERSION_V3\x10\x03"\x04\x08\x01\x10\x01\x42\x33Z1code.vegaprotocol.io/vega/protos/vega/commands/v1b\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "vega.commands.v1.transaction_pb2", globals()
)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = (
        b"Z1code.vegaprotocol.io/vega/protos/vega/commands/v1"
    )
    _TXVERSION._serialized_start = 2876
    _TXVERSION._serialized_end = 2959
    _INPUTDATA._serialized_start = 196
    _INPUTDATA._serialized_end = 2542
    _TRANSACTION._serialized_start = 2545
    _TRANSACTION._serialized_end = 2819
    _PROOFOFWORK._serialized_start = 2821
    _PROOFOFWORK._serialized_end = 2874
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vega/api/v1/core.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ... import vega_pb2 as vega_dot_vega__pb2
from ...events.v1 import events_pb2 as vega_dot_events_dot_v1_dot_events__pb2
from ...commands.v1 import (
    transaction_pb2 as vega_dot_commands_dot_v1_dot_transaction__pb2,
)
from protoc_gen_openapiv2.options import (
    annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2,
)


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x16vega/api/v1/core.proto\x12\x0bvega.api.v1\x1a\x0fvega/vega.proto\x1a\x1bvega/events/v1/events.proto\x1a"vega/commands/v1/transaction.proto\x1a.protoc-gen-openapiv2/options/annotations.proto"i\n\x1aPropagateChainEventRequest\x12\x14\n\x05\x65vent\x18\x01 \x01(\x0cR\x05\x65vent\x12\x17\n\x07pub_key\x18\x02 \x01(\tR\x06pubKey\x12\x1c\n\tsignature\x18\x03 \x01(\x0cR\tsignature"7\n\x1bPropagateChainEventResponse\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success"\xd7\x01\n\x18SubmitTransactionRequest\x12-\n\x02tx\x18\x01 \x01(\x0b\x32\x1d.vega.commands.v1.TransactionR\x02tx\x12>\n\x04type\x18\x02 \x01(\x0e\x32*.vega.api.v1.SubmitTransactionRequest.TypeR\x04type"L\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x0e\n\nTYPE_ASYNC\x10\x01\x12\r\n\tTYPE_SYNC\x10\x02\x12\x0f\n\x0bTYPE_COMMIT\x10\x03"\xa0\x01\n\x19SubmitTransactionResponse\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\x12\x17\n\x07tx_hash\x18\x02 \x01(\tR\x06txHash\x12\x12\n\x04\x63ode\x18\x03 \x01(\rR\x04\x63ode\x12\x12\n\x04\x64\x61ta\x18\x04 \x01(\tR\x04\x64\x61ta\x12\x10\n\x03log\x18\x05 \x01(\tR\x03log\x12\x16\n\x06height\x18\x06 \x01(\x03R\x06height"H\n\x17\x43heckTransactionRequest\x12-\n\x02tx\x18\x01 \x01(\x0b\x32\x1d.vega.commands.v1.TransactionR\x02tx"\x82\x01\n\x18\x43heckTransactionResponse\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\x12\x12\n\x04\x63ode\x18\x02 \x01(\rR\x04\x63ode\x12\x1d\n\ngas_wanted\x18\x03 \x01(\x03R\tgasWanted\x12\x19\n\x08gas_used\x18\x04 \x01(\x03R\x07gasUsed"\xbe\x01\n\x1bSubmitRawTransactionRequest\x12\x0e\n\x02tx\x18\x01 \x01(\x0cR\x02tx\x12\x41\n\x04type\x18\x02 \x01(\x0e\x32-.vega.api.v1.SubmitRawTransactionRequest.TypeR\x04type"L\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x0e\n\nTYPE_ASYNC\x10\x01\x12\r\n\tTYPE_SYNC\x10\x02\x12\x0f\n\x0bTYPE_COMMIT\x10\x03"\xa3\x01\n\x1cSubmitRawTransactionResponse\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\x12\x17\n\x07tx_hash\x18\x02 \x01(\tR\x06txHash\x12\x12\n\x04\x63ode\x18\x03 \x01(\rR\x04\x63ode\x12\x12\n\x04\x64\x61ta\x18\x04 \x01(\tR\x04\x64\x61ta\x12\x10\n\x03log\x18\x05 \x01(\tR\x03log\x12\x16\n\x06height\x18\x06 \x01(\x03R\x06height",\n\x1a\x43heckRawTransactionRequest\x12\x0e\n\x02tx\x18\x01 \x01(\x0cR\x02tx"\x85\x01\n\x1b\x43heckRawTransactionResponse\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\x12\x12\n\x04\x63ode\x18\x02 \x01(\rR\x04\x63ode\x12\x1d\n\ngas_wanted\x18\x03 \x01(\x03R\tgasWanted\x12\x19\n\x08gas_used\x18\x04 \x01(\x03R\x07gasUsed"\x14\n\x12GetVegaTimeRequest"3\n\x13GetVegaTimeResponse\x12\x1c\n\ttimestamp\x18\x01 \x01(\x03R\ttimestamp"\xa1\x01\n\x16ObserveEventBusRequest\x12\x30\n\x04type\x18\x01 \x03(\x0e\x32\x1c.vega.events.v1.BusEventTypeR\x04type\x12\x1b\n\tmarket_id\x18\x02 \x01(\tR\x08marketId\x12\x19\n\x08party_id\x18\x03 \x01(\tR\x07partyId\x12\x1d\n\nbatch_size\x18\x04 \x01(\x03R\tbatchSize"K\n\x17ObserveEventBusResponse\x12\x30\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x18.vega.events.v1.BusEventR\x06\x65vents"\x13\n\x11StatisticsRequest"M\n\x12StatisticsResponse\x12\x37\n\nstatistics\x18\x01 \x01(\x0b\x32\x17.vega.api.v1.StatisticsR\nstatistics"\xfb\x0b\n\nStatistics\x12!\n\x0c\x62lock_height\x18\x01 \x01(\x04R\x0b\x62lockHeight\x12%\n\x0e\x62\x61\x63klog_length\x18\x02 \x01(\x04R\rbacklogLength\x12\x1f\n\x0btotal_peers\x18\x03 \x01(\x04R\ntotalPeers\x12!\n\x0cgenesis_time\x18\x04 \x01(\tR\x0bgenesisTime\x12!\n\x0c\x63urrent_time\x18\x05 \x01(\tR\x0b\x63urrentTime\x12\x1b\n\tvega_time\x18\x06 \x01(\tR\x08vegaTime\x12)\n\x06status\x18\x07 \x01(\x0e\x32\x11.vega.ChainStatusR\x06status\x12 \n\x0ctx_per_block\x18\x08 \x01(\x04R\ntxPerBlock\x12(\n\x10\x61verage_tx_bytes\x18\t \x01(\x04R\x0e\x61verageTxBytes\x12\x37\n\x18\x61verage_orders_per_block\x18\n \x01(\x04R\x15\x61verageOrdersPerBlock\x12*\n\x11trades_per_second\x18\x0b \x01(\x04R\x0ftradesPerSecond\x12*\n\x11orders_per_second\x18\x0c \x01(\x04R\x0fordersPerSecond\x12#\n\rtotal_markets\x18\r \x01(\x04R\x0ctotalMarkets\x12*\n\x11total_amend_order\x18\x10 \x01(\x04R\x0ftotalAmendOrder\x12,\n\x12total_cancel_order\x18\x11 \x01(\x04R\x10totalCancelOrder\x12,\n\x12total_create_order\x18\x12 \x01(\x04R\x10totalCreateOrder\x12!\n\x0ctotal_orders\x18\x13 \x01(\x04R\x0btotalOrders\x12!\n\x0ctotal_trades\x18\x14 \x01(\x04R\x0btotalTrades\x12/\n\x13order_subscriptions\x18\x15 \x01(\rR\x12orderSubscriptions\x12/\n\x13trade_subscriptions\x18\x16 \x01(\rR\x12tradeSubscriptions\x12\x31\n\x14\x63\x61ndle_subscriptions\x18\x17 \x01(\rR\x13\x63\x61ndleSubscriptions\x12<\n\x1amarket_depth_subscriptions\x18\x18 \x01(\rR\x18marketDepthSubscriptions\x12\x37\n\x17positions_subscriptions\x18\x19 \x01(\rR\x16positionsSubscriptions\x12\x33\n\x15\x61\x63\x63ount_subscriptions\x18\x1a \x01(\rR\x14\x61\x63\x63ountSubscriptions\x12:\n\x19market_data_subscriptions\x18\x1b \x01(\rR\x17marketDataSubscriptions\x12(\n\x10\x61pp_version_hash\x18\x1c \x01(\tR\x0e\x61ppVersionHash\x12\x1f\n\x0b\x61pp_version\x18\x1d \x01(\tR\nappVersion\x12#\n\rchain_version\x18\x1e \x01(\tR\x0c\x63hainVersion\x12%\n\x0e\x62lock_duration\x18\x1f \x01(\x04R\rblockDuration\x12\x16\n\x06uptime\x18  \x01(\tR\x06uptime\x12\x19\n\x08\x63hain_id\x18! \x01(\tR\x07\x63hainId\x12K\n"market_depth_updates_subscriptions\x18" \x01(\rR\x1fmarketDepthUpdatesSubscriptions\x12\x1d\n\nblock_hash\x18# \x01(\tR\tblockHash\x12\x1b\n\tepoch_seq\x18$ \x01(\x04R\x08\x65pochSeq\x12(\n\x10\x65poch_start_time\x18% \x01(\tR\x0e\x65pochStartTime\x12*\n\x11\x65poch_expiry_time\x18& \x01(\tR\x0f\x65pochExpiryTime"\x18\n\x16LastBlockHeightRequest"\x91\x03\n\x17LastBlockHeightResponse\x12\x16\n\x06height\x18\x01 \x01(\x04R\x06height\x12\x12\n\x04hash\x18\x02 \x01(\tR\x04hash\x12\x33\n\x16spam_pow_hash_function\x18\x03 \x01(\tR\x13spamPowHashFunction\x12.\n\x13spam_pow_difficulty\x18\x04 \x01(\rR\x11spamPowDifficulty\x12\x41\n\x1espam_pow_number_of_past_blocks\x18\x05 \x01(\rR\x19spamPowNumberOfPastBlocks\x12\x42\n\x1fspam_pow_number_of_tx_per_block\x18\x06 \x01(\rR\x19spamPowNumberOfTxPerBlock\x12\x43\n\x1espam_pow_increasing_difficulty\x18\x07 \x01(\x08R\x1bspamPowIncreasingDifficulty\x12\x19\n\x08\x63hain_id\x18\x08 \x01(\tR\x07\x63hainId2\xf4\x06\n\x0b\x43oreService\x12\x62\n\x11SubmitTransaction\x12%.vega.api.v1.SubmitTransactionRequest\x1a&.vega.api.v1.SubmitTransactionResponse\x12h\n\x13PropagateChainEvent\x12\'.vega.api.v1.PropagateChainEventRequest\x1a(.vega.api.v1.PropagateChainEventResponse\x12M\n\nStatistics\x12\x1e.vega.api.v1.StatisticsRequest\x1a\x1f.vega.api.v1.StatisticsResponse\x12\\\n\x0fLastBlockHeight\x12#.vega.api.v1.LastBlockHeightRequest\x1a$.vega.api.v1.LastBlockHeightResponse\x12P\n\x0bGetVegaTime\x12\x1f.vega.api.v1.GetVegaTimeRequest\x1a .vega.api.v1.GetVegaTimeResponse\x12`\n\x0fObserveEventBus\x12#.vega.api.v1.ObserveEventBusRequest\x1a$.vega.api.v1.ObserveEventBusResponse(\x01\x30\x01\x12k\n\x14SubmitRawTransaction\x12(.vega.api.v1.SubmitRawTransactionRequest\x1a).vega.api.v1.SubmitRawTransactionResponse\x12_\n\x10\x43heckTransaction\x12$.vega.api.v1.CheckTransactionRequest\x1a%.vega.api.v1.CheckTransactionResponse\x12h\n\x13\x43heckRawTransaction\x12\'.vega.api.v1.CheckRawTransactionRequest\x1a(.vega.api.v1.CheckRawTransactionResponseBiZ,code.vegaprotocol.io/vega/protos/vega/api/v1\x92\x41\x38\x12\x1d\n\x0eVega core APIs2\x0bv0.63.0+dev\x1a\x13lb.testnet.vega.xyz*\x02\x01\x02\x62\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "vega.api.v1.core_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"Z,code.vegaprotocol.io/vega/protos/vega/api/v1\222A8\022\035\n\016Vega core APIs2\013v0.63.0+dev\032\023lb.testnet.vega.xyz*\002\001\002"
    _PROPAGATECHAINEVENTREQUEST._serialized_start = 169
    _PROPAGATECHAINEVENTREQUEST._serialized_end = 274
    _PROPAGATECHAINEVENTRESPONSE._serialized_start = 276
    _PROPAGATECHAINEVENTRESPONSE._serialized_end = 331
    _SUBMITTRANSACTIONREQUEST._serialized_start = 334
    _SUBMITTRANSACTIONREQUEST._serialized_end = 549
    _SUBMITTRANSACTIONREQUEST_TYPE._serialized_start = 473
    _SUBMITTRANSACTIONREQUEST_TYPE._serialized_end = 549
    _SUBMITTRANSACTIONRESPONSE._serialized_start = 552
    _SUBMITTRANSACTIONRESPONSE._serialized_end = 712
    _CHECKTRANSACTIONREQUEST._serialized_start = 714
    _CHECKTRANSACTIONREQUEST._serialized_end = 786
    _CHECKTRANSACTIONRESPONSE._serialized_start = 789
    _CHECKTRANSACTIONRESPONSE._serialized_end = 919
    _SUBMITRAWTRANSACTIONREQUEST._serialized_start = 922
    _SUBMITRAWTRANSACTIONREQUEST._serialized_end = 1112
    _SUBMITRAWTRANSACTIONREQUEST_TYPE._serialized_start = 473
    _SUBMITRAWTRANSACTIONREQUEST_TYPE._serialized_end = 549
    _SUBMITRAWTRANSACTIONRESPONSE._serialized_start = 1115
    _SUBMITRAWTRANSACTIONRESPONSE._serialized_end = 1278
    _CHECKRAWTRANSACTIONREQUEST._serialized_start = 1280
    _CHECKRAWTRANSACTIONREQUEST._serialized_end = 1324
    _CHECKRAWTRANSACTIONRESPONSE._serialized_start = 1327
    _CHECKRAWTRANSACTIONRESPONSE._serialized_end = 1460
    _GETVEGATIMEREQUEST._serialized_start = 1462
    _GETVEGATIMEREQUEST._serialized_end = 1482
    _GETVEGATIMERESPONSE._serialized_start = 1484
    _GETVEGATIMERESPONSE._serialized_end = 1535
    _OBSERVEEVENTBUSREQUEST._serialized_start = 1538
    _OBSERVEEVENTBUSREQUEST._serialized_end = 1699
    _OBSERVEEVENTBUSRESPONSE._serialized_start = 1701
    _OBSERVEEVENTBUSRESPONSE._serialized_end = 1776
    _STATISTICSREQUEST._serialized_start = 1778
    _STATISTICSREQUEST._serialized_end = 1797
    _STATISTICSRESPONSE._serialized_start = 1799
    _STATISTICSRESPONSE._serialized_end = 1876
    _STATISTICS._serialized_start = 1879
    _STATISTICS._serialized_end = 3410
    _LASTBLOCKHEIGHTREQUEST._serialized_start = 3412
    _LASTBLOCKHEIGHTREQUEST._serialized_end = 3436
    _LASTBLOCKHEIGHTRESPONSE._serialized_start = 3439
    _LASTBLOCKHEIGHTRESPONSE._serialized_end = 3840
    _CORESERVICE._serialized_start = 3843
    _CORESERVICE._serialized_end = 4727
# @@protoc_insertion_point(module_scope)



import copy
import csv
import json
import os
import pathlib
from dataclasses import dataclass
from random import random
from typing import Any, Dict, List, Optional, Tuple, Union

import numpy as np
from vega_sim.null_service import VegaServiceNull
from vega_sim.scenario.scenario import Scenario

from vega_sim.api.market import MarketConfig

PARAMETER_AMEND_WALLET = ("param", "amend")

FILE_PATTERN = "NETP_{param_name}_{param_value}.csv"
FILE_PATTERN_LOB = "NETP_{param_name}_{param_value}_LOB.csv"
OUTPUT_DIR = "parameter_results"


@dataclass
class Experiment:
    name: str

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
        }


@dataclass
class SingleParameterExperiment(Experiment):
    parameter_to_vary: str
    values: List[str]
    scenario: Scenario
    runs_per_scenario: int = 1
    additional_parameters_to_set: Optional[Dict[str, str]] = None
    data_extraction: List[Tuple] = None
    market_parameter: Optional[bool] = False

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "parameter_tested": self.parameter_to_vary,
            "tested_values": self.values,
            "scenario": self.scenario.__class__.__name__,
            "num_runs": self.runs_per_scenario,
            "additional_parameters": self.additional_parameters_to_set,
            "data_extraction": self.data_extraction,
        }


def _run_parameter_iteration(
    scenario: Scenario,
    parameter_to_vary: str,
    value: str,
    additional_parameters_to_set: Optional[Dict[str, str]] = None,
    random_state: Optional[np.random.RandomState] = None,
) -> Any:
    with VegaServiceNull(
        warn_on_raw_data_access=False,
        retain_log_files=True,
        run_with_console=False,
        transactions_per_block=100,
        use_full_vega_wallet=False,
    ) as vega:
        vega.create_wallet(*PARAMETER_AMEND_WALLET)
        vega.mint(
            PARAMETER_AMEND_WALLET[0],
            asset="VOTE",
            amount=1e4,
        )

        if additional_parameters_to_set is not None:
            for param, new_value in additional_parameters_to_set.items():
                vega.update_network_parameter(
                    PARAMETER_AMEND_WALLET[0], parameter=param, new_value=new_value
                )

        vega.update_network_parameter(
            PARAMETER_AMEND_WALLET[0], parameter=parameter_to_vary, new_value=value
        )

        res = scenario.run_iteration(vega=vega, random_state=random_state)

        return res


def _run_market_parameter_iteration(
    scenario: Scenario,
    parameter_to_vary: str,
    value: Union[str, int, float],
    random_state: Optional[np.random.RandomState],
) -> Any:

    with VegaServiceNull(
        warn_on_raw_data_access=False,
        retain_log_files=True,
        run_with_console=False,
        transactions_per_block=100,
        use_full_vega_wallet=False,
    ) as vega:

        market_config = MarketConfig("default")

        market_config.set(parameter_to_vary, value)

        res = scenario.run_iteration(
            vega=vega,
            random_state=random_state,
            market_config=market_config,
        )

        return res


def run_single_parameter_experiment(
    experiment: SingleParameterExperiment,
) -> Dict[str, List[Any]]:
    results = {}
    random_seeds = [
        np.random.RandomState(i) for i in range(experiment.runs_per_scenario)
    ]
    for value in experiment.values:
        results[value] = []
        for state in copy.deepcopy(random_seeds):
            if experiment.market_parameter:
                res = _run_market_parameter_iteration(
                    scenario=experiment.scenario,
                    parameter_to_vary=experiment.parameter_to_vary,
                    value=value,
                    random_state=state,
                )
            else:
                res = _run_parameter_iteration(
                    scenario=experiment.scenario,
                    parameter_to_vary=experiment.parameter_to_vary,
                    value=value,
                    random_state=state,
                )
            results[value].append(res)

    return results


def output_logs(
    results: Dict[str, List[List[Tuple[List[Any], Dict[str, Any]]]]],
    parameter_name: str,
    experiment: Experiment,
    file_pattern: Optional[str] = FILE_PATTERN,
    result_folder: Optional[str] = None,
    col_ordering: Optional[List[str]] = None,
    config_json: bool = True,
):
    result_folder = result_folder or OUTPUT_DIR
    final_folder = pathlib.Path(result_folder) / experiment.name
    os.makedirs(final_folder, exist_ok=True)

    if config_json:
        with open(final_folder / "run_config.json", "w") as f:
            json.dump(experiment.to_dict(), f, indent=4, sort_keys=True)

    if experiment.data_extraction is None:
        for value, result_list in results.items():
            file_path = final_folder / file_pattern.format(
                param_name=parameter_name, param_value=value
            )
        with open(file_path, "w") as f:
            csv_writer = csv.writer(f, delimiter=",")
            headers = (
                list(result_list[0][0].keys()) if col_ordering is None else col_ordering
            )
            csv_writer.writerow(["Iteration"] + headers)
            for i, res in enumerate(result_list):
                for row in res:
                    csv_writer.writerow([i] + [row[c] for c in headers])
    else:
        for value, result_list in results.items():
            for data_extraction in experiment.data_extraction:
                file_path = final_folder / data_extraction[0].format(
                    param_name=parameter_name, param_value=value
                )
                with open(file_path, "w") as f:
                    csv_writer = csv.writer(f, delimiter=",")
                    csv_writer.writerow(["Iteration"] + data_extraction[1])
                    for i, res in enumerate(result_list):
                        for row in res:
                            csv_writer.writerow(
                                [i] + [row[c] for c in data_extraction[1]]
                            )



import torch

from torch.distributions.normal import Normal


def lognorm_sample(mu: torch.Tensor, sigma: torch.Tensor):
    z = mu + sigma * torch.randn_like(sigma)
    sample = torch.exp(z)
    return z, sample


def lognorm_logprob(z: torch.Tensor, mu: torch.Tensor, sigma: torch.Tensor):
    """
    ENtropy of lognormal calculated using samples.
    We change the variable to calculate the expected value of the log-density of the lognormal
    """
    normal = Normal(loc=mu, scale=sigma)
    logprob = normal.log_prob(z) - z

    return logprob


def reg_policy(z: torch.Tensor, mu: torch.Tensor, sigma: torch.Tensor):
    """
    Regularisation term for the policy: D_KL(N(policy_parameters), N(0,1))

    """
    normal_policy = Normal(loc=mu, scale=sigma)
    normal_ref = Normal(loc=torch.zeros_like(mu), scale=torch.ones_like(sigma))

    d_kl = normal_policy.log_prob(z) - normal_ref.log_prob(z)
    return d_kl
